<?php 

/**
 * Plugin Name:          ELEX Choose Your Delivery Date
 * Description:          A plugin that helps customers choose a specific date for delivery.
 * Plugin URI:           https://elextensions.com/plugin/elex-woocommerce-choose-your-delivery-date-free/
 * Version:              1.2.1
 * WC requires at least: 2.6.0
 * WC tested up to:      8.6
 * Author:               ELEXtensions
 * Author URI:           https://elextensions.com
 * Text Domain:          elex-woocommerce-choose-your-delivery-date
 */

//is_plugin_active() function needs this file
require_once ABSPATH . 'wp-admin/includes/plugin.php';
define( 'ELEX_CD_VERSION', '1.0.0' );
//checking whether woocommerce is active
if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
	add_action( 'admin_notices', 'elex_cd_woocommerce_inactive_error' );
}


//Notify user if woocommerce is not active
function elex_cd_woocommerce_inactive_error() { 
	echo "<div class='notice notice-error'><p><b>woocommerce</b> must be active for <b>Custom Delivery Date</b> plugin to work.</p></div> ";
}

if ( ! defined( 'ELEX_CD_IMG_PATH' ) ) {
	define( 'ELEX_CD_IMG_PATH', plugin_dir_url( __FILE__ ) . 'resources/' );
}

//Loading css & js files(for front end)
add_action( 'wp_enqueue_scripts', 'elex_cd_load_assets' );
function elex_cd_load_assets() {
	wp_enqueue_style( 'elex_cd_plugin_styles' , plugin_dir_url( __FILE__ ) . 'resources/css/styles.css', ELEX_CD_VERSION , 'all' );
	wp_enqueue_style( 'elex_cd_flatpickr_styles' , plugin_dir_url( __FILE__ ) . 'resources/css/flatpickr.min.css', ELEX_CD_VERSION , 'all' );
	wp_enqueue_style( 'elex_cd_flatpickr_theme' , plugin_dir_url( __FILE__ ) . 'resources/css/theme_material_blue.css', ELEX_CD_VERSION , 'all' );
	wp_enqueue_script( 'elex_cd_flatpickr_scripts', plugin_dir_url( __FILE__ ) . 'resources/flatpickr_js/flatpickr.js', array( 'jquery' ), ELEX_CD_VERSION , false );
}

//Loading css & js files(for admin pages)
add_action( 'admin_enqueue_scripts', 'elex_cd_load_assets_admin' );
function elex_cd_load_assets_admin() {
	wp_enqueue_script( 'elex_cd_plugin_scripts', plugin_dir_url( __FILE__ ) . 'resources/admin_js/admin_scripts.js', array( 'jquery' ), ELEX_CD_VERSION , true );
	wp_enqueue_style( 'elex_cd_plugin_styles' , plugin_dir_url( __FILE__ ) . 'resources/admin_css/admin_styles.css', ELEX_CD_VERSION, 'all' );
}

//Adding 'Settings' and 'Premium plugins' links in admin plugins page under this plugin
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'elex_cd_create_settings_link' );
function elex_cd_create_settings_link( $links ) {
	$premuim_page_url  = 'https://elextensions.com/product-category/plugins/';
	$premuim_page_link = "<a href='$premuim_page_url'>" . __( 'ELEX Premium Plugins', 'elex-woocommerce-choose-your-delivery-date' ) . '</a>';
	array_unshift( $links, $premuim_page_link );
	$url           = esc_url( add_query_arg( 'page', 'wc-settings&tab=custom_date_label_settings', get_admin_url() . 'admin.php' ) );
	$settings_link = "<a href='$url'>" . __( 'Settings', 'elex-woocommerce-choose-your-delivery-date' ) . '</a>';
	array_unshift( $links, $settings_link );
	return $links;
}

//Adding a tab for our plugin's settings in woocommerce settings page
add_filter( 'woocommerce_settings_tabs_array', 'elex_cd_add_woocommerce_settings_tab', 99, 1 );
$tab_name = 'custom_date_label_settings';
function elex_cd_add_woocommerce_settings_tab( $tabs_array ) {
//Adding one more tab
	$tabs_array['custom_date_label_settings'] = __( 'ELEX Choose Delivery Date', 'elex-woocommerce-choose-your-delivery-date' );
	return $tabs_array;
}

//creating the sections part for our tab in the admin section
function elex_cd_get_sections() {
	$sections = array(
		''        => __( 'General Settings', 'elex-woocommerce-choose-your-delivery-date' ),
		'premium' => __( 'Related Products!', 'elex-woocommerce-choose-your-delivery-date' ),
	);
	return apply_filters( 'woocommerce_get_sections_elex_cd_custom_date', $sections );
}

//Displaying the sections part in the tab content
add_filter( 'woocommerce_sections_custom_date_label_settings', 'elex_cd_output_sections' );
function elex_cd_output_sections() {
	global $current_section;
	global $tab_name;
	$sections = elex_cd_get_sections();
	echo '<ul class="subsubsub">';
	$array_keys = array_keys( $sections );
	foreach ( $sections as $id => $label ) {
		echo '<li><a href="' . esc_html( admin_url( 'admin.php?page=wc-settings&tab=' . $tab_name . '&section=' . $id ) ) . '" class="' . ( esc_html( $current_section ) === $id ? 'current elex_cd_' . esc_html( $id ) : 'elex_cd_' . esc_html( $id ) ) . '">' . esc_html( $label ) . '</a>' . ( end( $array_keys ) === $id ? '' : ' |' ) . '</li>';
	}
	echo '</ul><br class="clear" />';
}

//Creating the tab content
function elex_cd_settings_list() {
	global $current_section;
	$settings = array();
	if ( 'premium' === $current_section ) {
		include_once 'includes/market.php';
	} else {
		$label    = get_option( 'custom_delivery_date_label_name', 'Request delivery on a specific date' );
		$settings = array(
			'label_section_title'           => array(
				'name' => __( 'ELEX Choose Delivery Date', 'elex-woocommerce-choose-your-delivery-date' ),
				'type' => 'title',
				'id'   => 'custom_delivery_date_settings_section_title',
			),
			'label_title'                   => array(
				'name'     => __( 'Checkout Field Label', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'     => 'text',
				'value'    => $label,
				'desc'     => __( 'Enter text to customize the checkout field label name.', 'elex-woocommerce-choose-your-delivery-date' ),
				'id'       => 'custom_delivery_date_label_name',
				'desc_tip' => true,
			),
			'label_section_end'             => array(
				'type' => 'sectionend',
				'id'   => 'custom_date_settings_list_end',
			),
			'minimum_delivery_days_section' => array(
				'name' => __( 'Minimum Delivery Days', 'elex-woocommerce-choose-your-delivery-date' ),
				'type' => 'title',
				'id'   => 'minimum_delivery_days_section',
			),
			'enable_minimum_delivery_days'  => array(
				'name'    => __( 'Set Minimum Delivery Days', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'    => 'checkbox',
				'default' => 'no',
				'desc'    => __( 'Enable', 'elex-woocommerce-choose-your-delivery-date' ),
				'id'      => 'enable_minimum_delivery_days',
			),
			'minimum_delivery_days'         => array(
				'name'  => __( 'Minimum days you take to deliver products', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'  => 'number',
				'value' => get_option( 'minimum_delivery_days' ),
				'desc'  => '',
				'id'    => 'minimum_delivery_days',
			),
			'minimum_days_section_end'      => array(
				'type' => 'sectionend',
				'id'   => 'minimum_days_section_end',
			),
			'holiday_section_title'         => array(
				'name' => __( 'Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type' => 'title',
				'id'   => 'delivery_date_holiday_settings_section',
			),
			'elex_cd_holiday_sunday'        => array(
				'name'          => __( 'Select Your Weekly Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Sunday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => 'start',
				'id'            => 'elex_cd_holiday_sunday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_monday'        => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Monday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => '',
				'id'            => 'elex_cd_holiday_monday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_tuesday'       => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Tuesday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => '',
				'id'            => 'elex_cd_holiday_tuesday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_wednesday'     => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Wednesday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => '',
				'id'            => 'elex_cd_holiday_wednesday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_thursday'      => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Thursday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => '',
				'id'            => 'elex_cd_holiday_thursday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_friday'        => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Friday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => '',
				'id'            => 'elex_cd_holiday_friday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_holiday_saturday'      => array(
				'name'          => __( 'Select Your Holidays', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'checkbox',
				'default'       => 'no',
				'desc'          => __( 'Saturday', 'elex-woocommerce-choose-your-delivery-date' ),
				'checkboxgroup' => 'end',
				'id'            => 'elex_cd_holiday_saturday',
				'class'         => 'elex_cd_holiday_week',
			),
			'elex_cd_note_to_customer'      => array(
				'name'          => __( 'Note To Customers', 'elex-woocommerce-choose-your-delivery-date' ),
				'type'          => 'textarea',
				'elex_cd_value' => get_option( 'elex_cd_note_to_customer', '' ),
				'placeholder'   => __( 'Set customer note here', 'elex-woocommerce-choose-your-delivery-date' ),
				'desc'          => __( 'You can notify customers about your weekly holidays or minimum delivery days etc.', 'elex-woocommerce-choose-your-delivery-daelex_cd_te' ),
				'id'            => 'elex_cd_note_to_customer',
				'desc_tip'      => true,
			),
			'holiday_section_end'           => array(
				'type' => 'sectionend',
				'id'   => 'custom_date_holiday_settings_end',
			),
		);
	}
	return apply_filters( 'elex_cd_custom_date_settings', $settings );
}

//Displaying the content in our tab
add_action( 'woocommerce_settings_tabs_custom_date_label_settings', 'elex_cd_custom_date_tab_content' );
function elex_cd_custom_date_tab_content() {
	woocommerce_admin_fields( elex_cd_settings_list() );
}

//Saving the changes in our settings tab
add_action( 'woocommerce_update_options_custom_date_label_settings', 'elex_cd_update_custom_date_settings' );
function elex_cd_update_custom_date_settings() {
	woocommerce_update_options( elex_cd_settings_list() );
}

//Adding the 'Requested Delivery Date' column on admin order page
add_filter( 'manage_edit-shop_order_columns', 'elex_cd_add_requested_date_column' );
function elex_cd_add_requested_date_column( $columns ) {
	$columns['elex_cd_custom_delivery'] = __( 'Requested Delivery Date', 'elex-woocommerce-choose-your-delivery-date' );
	return $columns;
}

//Showing the customer requested dates under the 'Requested Delivery Date' column
add_action( 'manage_shop_order_posts_custom_column', 'elex_cd_requested_date_column_content' );
function elex_cd_requested_date_column_content( $column ) {
	global $post;

	if (!$post && !$_GET['id']) {
		return;
	}
	if( isset( $_GET['id'] ) ){
		$post = get_post( $_GET['id'] );
	}

	if ( 'elex_cd_custom_delivery' === $column ) {
		$date = wc_get_order_item_meta( $post->ID, 'Requested_delivery_date' );
		if ( empty( $date ) ) {
			echo esc_html( __( 'No Request', 'elex-woocommerce-choose-your-delivery-date' ) );
		} else {
			echo esc_html( $date );
		}
	}
}
//Admin part ends here

//custom field creation on checkout page
add_action( 'woocommerce_after_checkout_billing_form', 'elex_cd_add_custom_date' );

function elex_cd_add_custom_date( $checkout ) {

	$label = get_option( 'custom_delivery_date_label_name', __( 'Request delivery on a specific date', 'elex-woocommerce-choose-your-delivery-date' ) );
	echo '<div id="my_custom_checkout_field">';
	$today              = gmdate( 'Y-m-d' );
	$current_weekday    = gmdate( 'w' );
	$today_array        = explode( '-', $today );
	$curr_year          = (int) $today_array[0];
	$curr_month         = (int) $today_array[1];
	$curr_date          = (int) $today_array[2];
	$min_attribute_date = $curr_date;
	$enable_m_d_days    = get_option( 'enable_minimum_delivery_days', 'no' );
	if ( strcmp( $enable_m_d_days, 'yes' ) === 0 ) {
		$min_del_days = get_option( 'minimum_delivery_days' );
		if ( ! is_numeric( $min_del_days ) ) {
			$min_del_days = 0;
		}
		$min_del = $min_del_days;
		if ( 0 !== $min_del_days && null !== $min_del_days ) {
			$elex_cd_get_holidays = elex_cd_get_holidays();
			$weekday              = $current_weekday;
			for ( $i = 1; $i <= $min_del_days; $i++ ) { 
				$weekday++;
				if ( 7 === $weekday ) {
					$weekday = 0;
				}
				if ( in_array( $weekday, $elex_cd_get_holidays, true ) ) {
					$min_del++;
				}
			}
		}
		$min_del_days        = $min_del;
		$min_attribute_date  = $curr_date + $min_del_days;
		$curr_month_max_days = gmdate( 't' );
		if ( $min_attribute_date > $curr_month_max_days ) {
			if ( 12 === $curr_month ) {
				$curr_year  = ++$curr_year;
				$curr_month = '1';
			} else {
				$curr_month = ++$curr_month;
			}
			$min_attribute_date = $min_attribute_date - $curr_month_max_days;
		}
	}
	$curr_month         = str_pad( $curr_month, 2, '0', STR_PAD_LEFT );
	$min_attribute_date = str_pad( $min_attribute_date, 2, '0', STR_PAD_LEFT );
	$final_date         = $curr_year . '-' . $curr_month . '-' . $min_attribute_date;
	woocommerce_form_field(
		'elex_cd_choose_date',
		array(
			'type'        => 'text',
			'class'       => array( 'my-field-class form-row-wide' ),
			'label'       => $label,
			'placeholder' => 'Select date for delivery',
			'required'    => false,
		),
		$checkout->get_value( 'elex_cd_choose_date' )
	);

	function getHolidayList() {
		$elex_cd_holidays      = elex_cd_get_holidays();
		$elex_cd_holiday_count = count( $elex_cd_holidays );
		$elex_cd_string        = '';
		for ( $z = 0; $z < $elex_cd_holiday_count; $z++ ) { 
			if ( $z !== $elex_cd_holiday_count - 1 ) {
				$elex_cd_string .= 'date.getDay() === ' . $elex_cd_holidays[ $z ] . '|| ';
			} else {
				$elex_cd_string .= 'date.getDay() === ' . $elex_cd_holidays[ $z ];
			}
		}
		if ( strlen( $elex_cd_string ) !== 0 ) {
			$elex_cd_disable_date_string = '
		        function(date) {
		            return (' . $elex_cd_string . ');
		        }
		    ';
			return $elex_cd_disable_date_string;
		}
	}
	?>
	<a id="elex_cd_date_clear" onclick="javascript:elex_cd_choose_date.value=''">clear</a>
	<script>
		flatpickr("#elex_cd_choose_date", {
			dateFormat: "Y-m-d",
			minDate: '<?php echo esc_html( $final_date ); ?>',
			'disable':[
				<?php echo esc_html( getHolidayList() ); ?>
			],
			"locale": {
				"firstDayOfWeek": 0
			}
		});

	</script>
	<input type="hidden" name="elex_cd_choose_date_nonce" value="<?php echo esc_html( wp_create_nonce( 'elex-cd-choose-date-nonce' ) ); ?>">
	</div>
	<?php
}

//getting array of holidays set by admin
function elex_cd_get_holidays() {
	$elex_cd_holidays = array();
	$sun              = get_option( 'elex_cd_holiday_sunday', 'no' );
	$mon              = get_option( 'elex_cd_holiday_monday', 'no' );
	$tue              = get_option( 'elex_cd_holiday_tuesday', 'no' );
	$wed              = get_option( 'elex_cd_holiday_wednesday', 'no' );
	$thu              = get_option( 'elex_cd_holiday_thursday', 'no' );
	$fri              = get_option( 'elex_cd_holiday_friday', 'no' );
	$sat              = get_option( 'elex_cd_holiday_saturday', 'no' );
	if ( strcmp( $sun, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 0, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $mon, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 1, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $tue, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 2, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $wed, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 3, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $thu, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 4, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $fri, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 5, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	if ( strcmp( $sat, 'yes' ) === 0 ) {
		array_push( $elex_cd_holidays, __( 6, 'elex-woocommerce-choose-your-delivery-date' ) );
	}
	return $elex_cd_holidays;
}

//Displaying holiday notice
add_action( 'woocommerce_after_checkout_billing_form', 'elex_cd_show_holidays' );
function elex_cd_show_holidays() {
	$elex_cd_note_to_customer = get_option( 'elex_cd_note_to_customer' );
	?>
	<span id='elex_cd_note_to_customer'> <?php echo esc_html( $elex_cd_note_to_customer ); ?></span>	
	<?php
}

//saving the date entered by the customer
add_action( 'woocommerce_checkout_update_order_meta', 'elex_cd_update_order_meta' );
function elex_cd_update_order_meta( $order_id ) {
	if ( isset( $_POST['elex_cd_choose_date'] ) ) {
		if ( isset( $_POST['elex_cd_choose_date_nonce'] ) ) {
			if ( wp_verify_nonce( sanitize_text_field( $_POST['elex_cd_choose_date_nonce'] ), 'elex-cd-choose-date-nonce' ) ) {
				wc_add_order_item_meta( $order_id, 'Requested_delivery_date', sanitize_text_field( $_POST['elex_cd_choose_date'] ), false );//updating the ordermeta table with the date entered by the customer
				if ( ! is_object( $order_id ) ) {
					$order = wc_get_order( $order_id );
				}
				$order->add_order_note( __( 'Requested delivery date is ', 'elex-woocommerce-choose-your-delivery-date' ) . sanitize_text_field( $_POST['elex_cd_choose_date'] ) );
				//Adding the order note
			}           
		}       
	}
}

// High performance order tables compatibility.
add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );


// review component
if ( ! function_exists( 'get_plugin_data' ) ) {
    require_once  ABSPATH . 'wp-admin/includes/plugin.php';
}
include_once __DIR__ . '/review_and_troubleshoot_notify/review-and-troubleshoot-notify-class.php';
$data                      = get_plugin_data( __FILE__ );
$data['name']              = $data['Name'];
$data['basename']          = plugin_basename( __FILE__ );
$data['rating_url']        = 'https://elextensions.com/plugin/elex-woocommerce-choose-your-delivery-date-free/#reviews';
$data['documentation_url'] = 'https://elextensions.com/knowledge-base/set-up-elex-woocommerce-choose-your-delivery-date-plugin/';
$data['support_url']       = 'https://wordpress.org/support/plugin/elex-choose-your-delivery-date/';

new \Elex_Review_Components( $data );